@@ -28,6 +28,25 @@
  *
  * Functions for understanding and manipulating argument vectors.
  *
+ * The few options explicitly handled by the client are processed in its
+ * main().  At the moment, this is just --help and --version, so this function
+ * never has to worry about them.
+ *
+ * We recognize two basic forms "distcc gcc ..." and "distcc ...", with no
+ * explicit compiler name.  This second one is used if you have a Makefile
+ * that can't manage two-word values for $CC; eventually it might support
+ * putting a link to distcc on your path as 'gcc'.  We call this second one an
+ * implicit compiler.
+ *
+ * We need to distinguish the two by working out whether the first argument
+ * "looks like" a compiler name or not.  I think the two cases in which we
+ * should assume it's implicit are "distcc -c hello.c" (starts with a hypen),
+ * and "distcc hello.c" (starts with a source filename.)
+ *
+ * In the case of implicit compilation "distcc --help" will always give you
+ * distcc's help, not gcc's, and similarly for --version.  I don't see much
+ * that we can do about that.
+ *
  * @todo We don't need to run the full argument scanner on the server, only
  * something simple to recognize input and output files.  That would perhaps
  * make the function simpler, and also mean that if argument recognizer bugs
@@ -35,11 +54,6 @@
  * the server.  An even better solution is to have the client tell the server
  * where to put the input and output files.
  *
- * @todo Detect arguments which ask the assembler to produce a listing in
- * addition to an object file.  These must be run locally.  This may be
- * complex, because it will look like -Wa,-alh or something.  We need to see
- * -Wa and then recursively examine the sub-arguments.
- *
  * @todo Perhaps make the argument parser driven by a data table.  (Would that
  * actually be clearer?)  Perhaps use regexps to recognize strings.
  *
@@ -74,7 +88,7 @@
 #include "util.h"
 #include "exitcode.h"
 #include "filename.h"
-
+#include "arg.h"
 
 static int dcc_argv_append(char *argv[], char *toadd)
 {
@@ -99,43 +113,13 @@ static void dcc_note_compiled(const char *input_file, const char *output_file)
  * Parse arguments, extract ones we care about, and also work out
  * whether it will be possible to distribute this invocation remotely.
  *
- * This is a little hard because the cc argument rules are pretty
- * complex, but the function still ought to be simpler than it already
- * is.
- *
- * The few options explicitly handled by the client are processed in its
- * main().  At the moment, this is just --help and --version, so this function
- * never has to worry about them.
- *
- * We recognize two basic forms "distcc gcc ..." and "distcc ...", with no
- * explicit compiler name.  This second one is used if you have a Makefile
- * that can't manage two-word values for $CC; eventually it might support
- * putting a link to distcc on your path as 'gcc'.  We call this second one an
- * implicit compiler.
- *
- * We need to distinguish the two by working out whether the first argument
- * "looks like" a compiler name or not.  I think the two cases in which we
- * should assume it's implicit are "distcc -c hello.c" (starts with a hypen),
- * and "distcc hello.c" (starts with a source filename.)
- *
- * In the case of implicit compilation "distcc --help" will always give you
- * distcc's help, not gcc's, and similarly for --version.  I don't see much
- * that we can do about that.
- *
- * XXX: It might be nice to split this up into several parsing runs.
- * However, that would possibly break handling of crazy command lines
- * like "gcc -o -o foo.c".
- *
- * @todo To fully support implicit compilers, we ought to call the child with
- * an environment variable set to protect against infinite (>1) recursion.
+ * This is a little hard because the cc argument rules are pretty complex, but
+ * the function still ought to be simpler than it already is.
  *
  * This code is called on both the client and the server, though they use the
- * results differently.  So it does not directly cause any actions, but only
- * updates fields in @p jobinfo.
+ * results differently.
  *
  * @returns 0 if it's ok to distribute this compilation, or an error code.
- *
- * @todo Fix this to use standard return codes.
  **/
 int dcc_scan_args(char *argv[], char **input_file, char **output_file,
                   char ***ret_newargv)
@@ -167,11 +151,12 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
             if (!strcmp(a, "-E")) {
                 rs_trace("-E call for cpp must be local");
                 return EXIT_DISTCC_FAILED;
-            } else if (!strcmp(a, "-MD") || !strcmp(a, "-MMD") || 
-                       !strcmp(a, "-MG") || !strcmp(a, "-MP")) {
-                /* -MD does *not* imply -E.  It will locally produce
-                    foo.d, and then we can remotely compile foo.o */
-                ;
+            } else if (!strcmp(a, "-MD") || !strcmp(a, "-MMD")) {
+                /* These two generate dependencies as a side effect.  They
+                 * should work with the way we call cpp. */
+            } else if (!strcmp(a, "-MG") || !strcmp(a, "-MP")) {
+                /* These just modify the behaviour of other -M* options and do
+                 * nothing by themselves. */
             } else if (!strcmp(a, "-MF") || !strcmp(a, "-MT") || 
                        !strcmp(a, "-MQ")) {
                 /* as above but with extra argument */
@@ -185,6 +170,18 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
                     to distribute it even if we could. */
                 rs_trace("%s implies -E (maybe) and must be local", a);
                 return EXIT_DISTCC_FAILED;
+            } else if (str_startswith("-Wa,", a)) {
+                /* Options passed through to the assembler.  The only one we
+                 * need to handle so far is -al=output, which directs the
+                 * listing to the named file and cannot be remote.  Parsing
+                 * all the options would be complex since you can give several
+                 * comma-separated assembler options after -Wa, but looking
+                 * for '=' should be safe. */
+                if (strchr(a, '=')) {
+                    rs_trace("%s needs to write out assembly listings and must be local",
+                         a);
+                    return EXIT_DISTCC_FAILED;
+                }
             } else if (!strcmp(a, "-S")) {
                 seen_opt_s = 1;
             } else if (!strcmp(a, "-fprofile-arcs")
@@ -224,8 +221,8 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
         }
     }
 
-    /* TODO: ccache has the heuristic of ignoring arguments that do
-     * not exist when looking for the input file; that's possibly
+    /* TODO: ccache has the heuristic of ignoring arguments that are not
+     * extant files when looking for the input file; that's possibly
      * worthwile.  Of course we can't do that on the server. */
 
     if (!seen_opt_c && !seen_opt_s) {
@@ -283,69 +280,6 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
 }
 
 
-int
-dcc_argv_len (char **a)
-{
-  int i;
-
-  for (i = 0; a[i]; i++)
-    ;
-  return i;
-}
-
-/**
- * Adds @p delta extra NULL arguments, to allow for adding more
- * arguments later.
- **/
-int dcc_shallowcopy_argv(char **from, /*@-nullstate@*/ char ***out, int delta)
-{
-    /*@null@*/ char **b;
-    int l, i;
-
-    l = dcc_argv_len(from);
-    b = malloc((l+1+delta) * (sizeof from[0]));
-    if (b == NULL) {
-        rs_log_error("failed to allocate copy of argv");
-        exit(EXIT_OUT_OF_MEMORY);
-    }
-    for (i = 0; i < l; i++) {
-        b[i] = from[i];
-    }
-    b[l] = NULL;
-    
-    *out = b;
-
-    return 0;
-}
-
-
-/**
- * Adds @p delta extra NULL arguments, to allow for adding more
- * arguments later.
- **/
-int dcc_deepcopy_argv(char **from, /*@-nullstate@*/ char ***out)
-{
-    char **b;
-    int i, l;
-
-    l = dcc_argv_len(from);
-    
-    l = dcc_argv_len(from);
-    b = malloc((l+1) * (sizeof from[0]));
-    if (b == NULL) {
-        rs_log_error("failed to allocate copy of argv");
-        exit(EXIT_OUT_OF_MEMORY);
-    }
-
-    for (i = 0; i < l; i++) 
-        b[i] = strdup(from[i]);
-    b[l] = NULL;
-
-    *out = b;
-
-    return 0;
-}
-
 
 /**
  * Used to change "-c" or "-S" to "-E", so that we get preprocessed
@@ -389,16 +323,23 @@ int dcc_set_output(char **a, char *ofname)
     int i;
     
     for (i = 0; a[i]; i++) 
-        if (0==strcmp(a[i], "-o")  &&  a[i+1] != NULL) {
-            rs_trace("changed output from \"%s\" to \"%s\"", a[i+1], ofname);
+        if (0 == strcmp(a[i], "-o")  &&  a[i+1] != NULL) {
+            rs_trace("changed output from \"%s\" to \"%s\"", a[i+1],
+                     ofname);
             a[i+1] = ofname;
             dcc_trace_argv("command after", a);
             return 0;
+        } else if (0 == strncmp(a[i], "-o", 2)) {
+            char *newptr;
+
+            if (asprintf(&newptr, "-o%s", ofname) == -1) {
+                rs_fatal("failed to allocate space for output parameter");
+            }
+            
+            a[i] = newptr;
+            dcc_trace_argv("command after", a);
+            return 0;
         }
-    
-    /* TODO: Handle -ofoo.o.  gcc supports it, though it seems to be
-     * rarely used.  At the moment we probably just fail to distribute
-     * these commands. */
 
     rs_log_error("failed to find \"-o\"");
     return EXIT_DISTCC_FAILED;
@@ -429,49 +370,3 @@ int dcc_set_input(char **a, char *ifname)
     return EXIT_DISTCC_FAILED;
 }
 
-
-/**
- * Convert an argv array to printable form for debugging output.
- *
- * @note The result is not necessarily properly quoted for passing to
- * shells.
- *
- * @return newly-allocated string containing representation of
- * arguments.
- **/
-char *dcc_argv_tostr(char **a)
-{
-    int l, i;
-    char *s, *ss;
-    
-    /* calculate total length */
-    for (l = 0, i = 0; a[i]; i++) {
-        l += strlen(a[i]) + 3;  /* two quotes and space */
-    }
-
-    ss = s = malloc((size_t) l + 1);
-    if (!s) {
-        rs_log_crit("failed to allocate %d bytes", l+1);
-        exit(EXIT_OUT_OF_MEMORY);
-    }
-    
-    for (i = 0; a[i]; i++) {
-        /* kind of half-assed quoting; won't handle strings containing
-         * quotes properly, but good enough for debug messages for the
-         * moment. */
-        int needs_quotes = (strpbrk(a[i], " \t\n\"\';") != NULL);
-        if (i)
-            *ss++ = ' ';
-        if (needs_quotes)
-            *ss++ = '"';
-        strcpy(ss, a[i]);
-        ss += strlen(a[i]);
-        if (needs_quotes)
-            *ss++ = '"';
-    }
-    *ss = '\0';
-
-    return s;
-}
-
-
