@@ -101,7 +101,7 @@ static int dcc_connect_timed(int fd,
         goto out_failed;
     }
     
-    if ((ret = dcc_select_for_read(fd)))
+    if ((ret = dcc_select_for_write(fd, dcc_connect_timeout)))
         goto out_failed;
 
     /* If the connection failed we may not yet have noticed the error; we'd
