@@ -88,7 +88,7 @@ static int dcc_x_argv(int fd, char **argv)
 static int dcc_send_job(int net_fd,
                         char **argv,
                         pid_t cpp_pid,
-                        unsigned *status,
+                        int *status,
                         const char *cpp_fname,
                         struct dcc_hostdef *host)
 {
@@ -149,7 +149,7 @@ static int dcc_send_job(int net_fd,
 int dcc_send_job_corked(int net_fd,
 			char **argv,
 			pid_t cpp_pid,
-                        unsigned *status,
+                        int *status,
 			const char *cpp_fname,
                         struct dcc_hostdef *host)
 {
@@ -190,16 +190,22 @@ int dcc_r_result_header(int ifd,
 }
 
 
-int dcc_r_cc_status(int ifd, unsigned *status)
+int dcc_r_cc_status(int ifd, int *status)
 {
-    return dcc_r_token_int(ifd, "STAT", status);
+    unsigned u_status;
+    int ret;
+    
+    ret = dcc_r_token_int(ifd, "STAT", &u_status);
+    *status = u_status;
+    return ret;
 }
 
 
 /**
  * The second half of the client protocol: retrieve all results from the server.
  **/
-int dcc_retrieve_results(int net_fd, unsigned *status,
+int dcc_retrieve_results(int net_fd,
+                         int *status,
                          const char *output_fname,
                          struct dcc_hostdef *host)
 {
