@@ -111,7 +111,9 @@ static int dcc_compile_local(char *argv[],
  * the compiler itself fails.  If either the compiler or preprocessor fails,
  * @p status is guaranteed to hold a failure value.
  **/
-static int dcc_build_somewhere(char *argv[], int sg_level, unsigned *status)
+static int dcc_build_somewhere(char *argv[],
+                               int sg_level,
+                               int *status)
 {
     char *input_fname = NULL, *output_fname, *cpp_fname;
     char **argv_stripped;
@@ -193,8 +195,8 @@ static int dcc_build_somewhere(char *argv[], int sg_level, unsigned *status)
 
 
 int dcc_build_somewhere_timed(char *argv[],
-                                     int sg_level,
-                                     int *status)
+                              int sg_level,
+                              int *status)
 {
     struct timeval before, after, delta;
     int ret;
