@@ -130,13 +130,30 @@ static int dcc_should_be_inetd(void)
 }
 
 
+static int dcc_setup_daemon_path(void)
+{
+    int ret;
+    const char *path;
+
+    if ((path = getenv("DISTCCD_PATH")) != NULL) {
+        if ((ret = dcc_set_path(path)))
+            return ret;
+
+        return 0;
+    } else {
+        path = getenv("PATH");
+        rs_log_info("daemon's PATH is %s", path ? path : "(NULL)");
+        return 0;
+    }
+}
+
+
 /**
  * distcc daemon.  May run from inetd, or standalone.  Accepts
  * requests from clients to compile files.
  **/
 int main(int argc, char *argv[])
 {
-    const char *requested_path;
     int ret;
 
     dcc_setup_startup_log();
@@ -146,6 +163,13 @@ int main(int argc, char *argv[])
 
     if ((ret = dcc_set_lifetime()) != 0)
         dcc_exit(ret);
+    
+    /* do this before giving away root */
+    if (nice(opt_niceness) == -1) {
+        rs_log_warning("nice %d failed: %s", opt_niceness,
+                       strerror(errno));
+        /* continue anyhow */
+    }
 
     if ((ret = dcc_discard_root()) != 0)
         dcc_exit(ret);
@@ -154,20 +178,18 @@ int main(int argc, char *argv[])
      * the right ownership. */
     dcc_setup_real_log();
 
-    /* Do everything from temporary directory.  Makes things more
-     * reproducible; avoids tempdir itself being deleted; allows start
-     * directory to be unmounted, ... */
-    if ((ret = dcc_enter_tempdir()))
+    /* Do everything from root directory.  Allows start directory to be
+     * unmounted, should make accidental writing of local files cause a
+     * failure... */
+    if (chdir("/") == -1) {
+        rs_log_error("failed to chdir to /: %s", strerror(errno));
+        ret = EXIT_IO_ERROR;
         goto out;
-     
-    if ((requested_path = getenv("DISTCCD_PATH")) != NULL) {
-        if ((ret = dcc_set_path(requested_path)))
-            goto out;
-    } else {
-        if ((ret = dcc_trim_path(NULL)))
-            goto out;
     }
 
+    if ((ret = dcc_setup_daemon_path()))
+        goto out;
+
     if (dcc_should_be_inetd())
         ret = dcc_inetd_server();
     else
@@ -252,8 +274,6 @@ static int dcc_inetd_server(void)
     
     dcc_log_daemon_started("inetd server");
 
-    /* XXX: ssh seems to open us using a unix domain socket.  Is that always
-     * the case? */
     if ((getpeername(STDIN_FILENO, psa, &len) == -1)) {
         rs_log_warning("failed to get peer name: %s", strerror(errno));
         len = 0;
