@@ -120,11 +120,12 @@ int dcc_recursion_safeguard(void);
 int dcc_send_job_corked(int net_fd,
 			char **argv,
 			pid_t cpp_pid,
-			unsigned *status,
+                        int *status,
 			const char *cpp_fname,
                         struct dcc_hostdef *);
 
-int dcc_retrieve_results(int net_fd, unsigned *status, const char *output_fname,
+int dcc_retrieve_results(int net_fd,
+                         int *status, const char *output_fname,
                          struct dcc_hostdef *);
 
 /* climasq.c */
