@@ -286,7 +286,7 @@ static int sys_wait4(pid_t pid, int *status, int options, struct rusage *rusage)
  * waits all the time.
  **/
 int dcc_collect_child(const char *what, pid_t pid,
-                      unsigned *wait_status)
+                      int *wait_status)
 {
     struct rusage ru;
     pid_t ret_pid;
@@ -326,8 +326,12 @@ int dcc_collect_child(const char *what, pid_t pid,
  *
  * @returns 0 if the command succeeded; 128+SIGNAL if it stopped on a
  * signal; otherwise the command's exit code.
+ *
+ * @todo It would be nice to show the input filename here, however when we're
+ * invoked by ccache our input filename may not be meaningful to the user.
+ * Therefore I'm going to leave it out for now.
  **/
-int dcc_critique_status(unsigned s,
+int dcc_critique_status(int status,
                         const char *command,
                         struct dcc_hostdef *host,
                         int verbose)
@@ -341,31 +345,35 @@ int dcc_critique_status(unsigned s,
     else
         logmode = RS_LOG_INFO | RS_LOG_NONAME;
     
-    if (WIFSIGNALED(s)) {
+    if (WIFSIGNALED(status)) {
 #ifdef HAVE_STRSIGNAL
         rs_log(logmode,
                "%s on %s:%s %s",
                command, host->hostdef_string,
-               strsignal(WTERMSIG(s)),
-               WCOREDUMP(s) ? " (core dumped)" : "");
+               strsignal(WTERMSIG(status)),
+               WCOREDUMP(status) ? " (core dumped)" : "");
 #else
         rs_log(logmode,
                "%s on %s terminated by signal %d%s",
                command, host->hostdef_string,
-               WTERMSIG(s),
-               WCOREDUMP(s) ? " (core dumped)" : "");
+               WTERMSIG(status),
+               WCOREDUMP(status) ? " (core dumped)" : "");
 #endif
         /* Unix convention is to return 128+signal when a subprocess crashes. */
-        return 128 + WTERMSIG(s);
-    } else if (WEXITSTATUS(s)) {
+        return 128 + WTERMSIG(status);
+    } else if (WEXITSTATUS(status) == 1) {
+        /* Normal failure gives exit code 1, so handle that specially */
+        rs_log(logmode, "%s on %s failed", command, host->hostdef_string);
+        return WEXITSTATUS(status);
+    } else if (WEXITSTATUS(status)) {
         /* This is a tough call; we don't really want to clutter the client's
          * error stream, but if we don't say where the compilation failed then
          * people may find it hard to work things out. */
 
         rs_log(logmode,
                "%s on %s failed with exit code %d",
-               command, host->hostdef_string, WEXITSTATUS(s));
-        return WEXITSTATUS(s);
+               command, host->hostdef_string, WEXITSTATUS(status));
+        return WEXITSTATUS(status);
     } else {
         rs_log(RS_LOG_INFO|RS_LOG_NONAME,
                "%s on %s completed ok", command, host->hostdef_string);
