@@ -20,6 +20,9 @@
  * USA
  */
 
+#ifndef _DISTCC_EXITCODE_H
+#define _DISTCC_EXITCODE_H
+
 /**
  * @file
  *
@@ -52,3 +55,6 @@ enum dcc_exitcode {
     EXIT_NO_HOSTS                 = 116,
     EXIT_GONE                     = 117, /**< No longer relevant */
 };
+
+
+#endif /* _DISTCC_EXITCODE_H */
