@@ -58,9 +58,9 @@ int dcc_show_version(const char *prog)
     /* The "built" message is the time this file was built, which may not be
      * completely accurate for the program as a whole unless you do "make
      * clean" */
-    printf("%s %s %s (protocol %d) (default port %d)\n"
+    printf("%s %s %s (protocols 1 and 2) (default port %d)\n"
            "  built %s %s\n",
-           prog, PACKAGE_VERSION, GNU_HOST, PROTO_VER, DISTCC_DEFAULT_PORT,
+           prog, PACKAGE_VERSION, GNU_HOST, DISTCC_DEFAULT_PORT,
            __DATE__, __TIME__);
     return 0;
 }
