@@ -34,12 +34,15 @@
  *
  * The grammar of this variable is, informally:
  *
- * DISTCC_HOSTS = HOST ...
- * HOSTID = SSH_HOST | TCP_HOST | OLDSTYLE_TCP_HOST
- * SSH_HOST = USER@HOSTID[/MUL][:COMMAND]
- * TCP_HOST = HOSTID[:PORT][/MUL]
- * OLDSTYLE_TCP_HOST = HOSTID[/MUL][:PORT]
- * HOSTID = HOSTNAME | IPV4 | '['IPV6']'
+  DISTCC_HOSTS = HOSTSPEC ...
+  HOSTSPEC = LOCAL_HOST | SSH_HOST | TCP_HOST | OLDSTYLE_TCP_HOST
+  LOCAL_HOST = localhost[/LIMIT]
+  SSH_HOST = [USER]@HOSTID[/LIMIT][:COMMAND][OPTIONS]
+  TCP_HOST = HOSTID[:PORT][/LIMIT][OPTIONS]
+  OLDSTYLE_TCP_HOST = HOSTID[/LIMIT][:PORT][OPTIONS]
+  HOSTID = HOSTNAME | IPV4
+  OPTIONS = ,OPTION[OPTIONS]
+  OPTION = lzo
  *
  * Any amount of whitespace may be present between hosts.
  *
@@ -195,6 +198,37 @@ static int dcc_parse_multiplier(const char **psrc, struct dcc_hostdef *hostdef)
     return 0;
 }
 
+
+/**
+ * Parse an optionally present option string.
+ *
+ * At the moment the only option we have is "lzo" for compression, so this is
+ * pretty damn simple.
+ **/
+static int dcc_parse_options(const char **psrc,
+                             struct dcc_hostdef *host)
+{
+    const char *started = *psrc, *p = *psrc;
+
+    while (p[0] == ',') {
+        p++;
+        if (str_startswith("lzo", p)) {
+            rs_trace("got LZO option");
+            host->protover = DCC_VER_2;
+            host->compr = DCC_COMPRESS_LZO1X;
+            p += 3;
+        } else {
+            rs_log_warning("unrecognized option in host specification %s",
+                           started);
+            return EXIT_BAD_HOSTSPEC;
+        }
+    }
+
+    *psrc = p;
+    
+    return 0;
+}
+
 static int dcc_parse_ssh_host(struct dcc_hostdef *hostdef,
                               const char *token_start)
 {
@@ -211,7 +245,7 @@ static int dcc_parse_ssh_host(struct dcc_hostdef *hostdef,
 
     token++;
 
-    if ((ret = dcc_dup_part(&token, &hostdef->hostname, "/: \t\n\f")) != 0)
+    if ((ret = dcc_dup_part(&token, &hostdef->hostname, "/: \t\n\f,")) != 0)
         return ret;
 
     if (!hostdef->hostname) {
@@ -225,10 +259,13 @@ static int dcc_parse_ssh_host(struct dcc_hostdef *hostdef,
 
     if (token[0] == ':') {
         token++;
-        if ((ret = dcc_dup_part(&token, &hostdef->ssh_command, " \t\n\f")) != 0)
+        if ((ret = dcc_dup_part(&token, &hostdef->ssh_command, " \t\n\f,")))
             return ret;
     }
     
+    if ((ret = dcc_parse_options(&token, hostdef)))
+        return ret;
+
     hostdef->mode = DCC_MODE_SSH;
     return 0;
 }
@@ -240,7 +277,7 @@ static int dcc_parse_tcp_host(struct dcc_hostdef *hostdef,
     int ret;
     const char *token = token_start;
     
-    if ((ret = dcc_dup_part(&token, &hostdef->hostname, "/: \t\n\f")) != 0)
+    if ((ret = dcc_dup_part(&token, &hostdef->hostname, "/: \t\n\f,")))
         return ret;
 
     if (!hostdef->hostname) {
@@ -259,7 +296,7 @@ static int dcc_parse_tcp_host(struct dcc_hostdef *hostdef,
         token++;
 
         hostdef->port = strtol(token, &tail, 10);
-        if (*tail != '\0' && !isspace(*tail) && *tail != '/') {
+        if (*tail != '\0' && !isspace(*tail) && *tail != '/' && *tail != ',') {
             rs_log_error("invalid tcp port specification in \"%s\"", token);
             return EXIT_BAD_HOSTSPEC; 
         } else {
@@ -270,6 +307,9 @@ static int dcc_parse_tcp_host(struct dcc_hostdef *hostdef,
     if ((ret = dcc_parse_multiplier(&token, hostdef)) != 0)
         return ret;
 
+    if ((ret = dcc_parse_options(&token, hostdef)))
+        return ret;
+
     hostdef->mode = DCC_MODE_TCP;
     return 0;
 }
@@ -358,6 +398,9 @@ int dcc_parse_hosts(const char *where, const char *source_name,
         /* Default task limit.  A bit higher than the local limit to allow for
          * some files in transit. */
         curr->n_slots = 4;
+
+        curr->protover = DCC_VER_1; /* default */
+        curr->compr = DCC_COMPRESS_NONE;
             
         has_at = (memchr(token_start, '@', token_len) != NULL);
 
