@@ -26,27 +26,15 @@
  *
  * Common low-level IO utilities.
  *
- * The pump_* methods are for doing bulk transfer of the entire
- * contents of a file to or from the socket.  We have various
- * different implementations to suit different circumstances.
- *
  * This code is not meant to know about our protocol, only to provide
  * a more comfortable layer on top of Unix IO.
  *
  * @todo Perhaps also add a method of copying that truncates and mmaps
  * the destination file, and then writes directly into it.
  *
- * @todo Perhaps also add a pump method that mmaps the input file, and
- * writes from there to the output file.
- **/
-
-/*
- * TODO: Perhaps write things out using writev() to reduce the number
+ * @todo Perhaps write things out using writev() to reduce the number
  * of system calls, and the risk of small packets when not using
  * TCP_CORK.
- *
- * TODO: Check for every call to read(), write(), and other long
- * system calls.  Make sure they handle EINTR appropriately.
  */
 
 #include "config.h"
@@ -77,9 +65,19 @@
 
 
 
-const int timeout = 15;          /* seconds */
+const int dcc_connect_timeout = 15; /* seconds */
 
-int dcc_select_for_read(int fd)
+/** Timeout for all IO other than opening connections.  Much longer, because
+ * compiling files can take a long time. **/
+const int dcc_io_timeout = 300; /* seconds */
+
+
+/**
+ * @todo Perhaps only apply the timeout for initial connections, not when
+ * doing regular IO.
+ **/
+int dcc_select_for_read(int fd,
+                        int timeout)
 {
     fd_set fds;
     int rs;
@@ -98,9 +96,10 @@ int dcc_select_for_read(int fd)
          * it's no big deal.
          */
 
-        rs_trace("select for read on fd%d", fd);
-        rs = select(fd+1, NULL, &fds, NULL, &tv);
+        rs_trace("select for read on fd%d for %ds", fd, (int) tv.tv_sec);
+        rs = select(fd+1, &fds, NULL, NULL, &tv);
         if (rs == -1 && errno == EINTR) {
+            rs_trace("select was interrupted");
             continue;
         } else if (rs == -1) {
             rs_log_error("select() failed: %s", strerror(errno));
@@ -118,93 +117,34 @@ int dcc_select_for_read(int fd)
 }
 
 
-/* TODO: Handle EINTR */
-int dcc_select_for_write(int fd)
+
+int dcc_select_for_write(int fd, int timeout)
 {
     fd_set fds;
-    FD_ZERO(&fds);
-    FD_SET(fd, &fds);
-    rs_trace("select for write on fd%d", fd);
-    if (select(fd + 1, NULL, &fds, &fds, NULL) == -1) {
-        rs_log_error("select failed: %s", strerror(errno));
-        return EXIT_IO_ERROR;
-    }
-    return 0;
-}
+    int rs;
 
+    struct timeval tv;
 
-/**
- * Copy @p n bytes from @p ifd to @p ofd.
- *
- * Does not use sendfile(), so either one may be a socket.
- *
- * In the current code at least one of the files will always be a regular
- * (disk) file, even though it may not be mmapable.  That should mean that
- * writes to it will always complete immediately.  That in turn means that on
- * each pass through the main loop we ought to either completely fill our
- * buffer, or completely drain it, depending on which one is the disk.
- *
- * In future we may put back the ability to feed the compiler from a fifo, in
- * which case it may be that the writes don't complete.
- *
- * We might try selecting on both buffers and handling whichever is ready.
- * This would require some approximation to a circular buffer though, which
- * might be more complex.
- *
- * However, don't overoptimize this because it has to be replaced to allow for
- * compression.
- **/
-int dcc_pump_readwrite(int ofd, int ifd, size_t n)
-{
-    static char buf[262144];    /* we're not recursive */
-    char *p;
-    ssize_t r_in, r_out, wanted;
-    int ret;
+    tv.tv_sec = timeout;
+    tv.tv_usec = 0;
 
-    while (n > 0) {
-        wanted = (n > sizeof buf) ? (sizeof buf) : n;
-        r_in = read(ifd, buf, (size_t) wanted);
-
-        if (r_in == -1 && errno == EAGAIN) {
-            if ((ret = dcc_select_for_read(ifd)) != 0)
-                return ret;
-            else
-                continue;
-        } else if (r_in == -1) {
-            rs_log_error("failed to read %ld bytes: %s",
-                         (long) wanted, strerror(errno));
-            return EXIT_IO_ERROR;
-        } else if (r_in == 0) {
-            rs_log_error("unexpected eof on fd%d", ifd);
-            return EXIT_IO_ERROR;
-        }
+    while (1) {
+        FD_ZERO(&fds);
+        FD_SET(fd, &fds);
+        rs_trace("select for write on fd%d", fd);
         
-        n -= r_in;
-        p = buf;
+        rs = select(fd + 1, NULL, &fds, &fds, &tv);
 
-        /* We now have r_in bytes waiting to go out, starting at p.  Keep
-         * going until they're all written out. */
-
-        while (r_in > 0) {
-            r_out = write(ofd, p, (size_t) r_in);
-            
-            if (r_out == -1 && errno == EAGAIN) {
-                if ((ret = dcc_select_for_write(ofd)) != 0)
-                    return ret;
-                else
-                    continue;
-            }
-            
-            if (r_out == -1  ||  r_out == 0) {
-                rs_log_error("failed to write: %s", strerror(errno));
-                return EXIT_IO_ERROR;
-            }
-            r_in -= r_out;
-            p += r_out;
+        if (rs == -1 && errno == EINTR) {
+            rs_trace("select was interrupted");
+            continue;
+        } else if (rs == -1) {
+            rs_log_error("select failed: %s", strerror(errno));
+            return EXIT_IO_ERROR;
+        } else {
+            return 0;
         }
     }
-
-    return 0;
 }
 
 
@@ -221,10 +161,12 @@ int dcc_readx(int fd, void *buf, size_t len)
 	r = read(fd, buf, len);
 
         if (r == -1 && errno == EAGAIN) {
-            if ((ret = dcc_select_for_read(fd)))
+            if ((ret = dcc_select_for_read(fd, dcc_io_timeout)))
                 return ret;
             else
                 continue;
+        } else if (r == -1 && errno == EAGAIN) {
+            continue;
         } else if (r == -1) {
 	    rs_log_error("failed to read: %s", strerror(errno));
 	    return EXIT_IO_ERROR;
@@ -256,10 +198,12 @@ int dcc_writex(int fd, const void *buf, size_t len)
         r = write(fd, buf, len);
 
         if (r == -1 && errno == EAGAIN) {
-            if ((ret = dcc_select_for_write(fd)))
+            if ((ret = dcc_select_for_write(fd, dcc_io_timeout)))
                 return ret;
             else
                 continue;
+        } else if (r == -1 && errno == EINTR) {
+            continue;
         } else if (r == -1) {
             rs_log_error("failed to write: %s", strerror(errno));
             return EXIT_IO_ERROR;
