@@ -25,7 +25,6 @@ int dcc_writex(int fd, const void *buf, size_t len);
 int dcc_r_token(int ifd, char *token);
 
 int dcc_readx(int fd, void *buf, size_t len);
-int dcc_pump_readwrite(int ofd, int ifd, size_t n);
 int dcc_pump_sendfile(int ofd, int ifd, size_t n);
 int dcc_r_str_alloc(int fd, int len, char **buf);
 
@@ -33,9 +32,25 @@ int tcp_cork_sock(int fd, int corked);
 int dcc_close(int fd);
 
 
-int dcc_select_for_write(int fd);
-int dcc_select_for_read(int fd);
+int dcc_select_for_write(int fd, int timeout);
+int dcc_select_for_read(int fd, int timeout);
 
 /* loadfile.c */
-int dcc_load_file(const char *filename,
-		  char **retbuf);
+int dcc_load_file_string(const char *filename,
+                         char **retbuf);
+
+
+extern const int dcc_connect_timeout, dcc_io_timeout;
+
+
+/* pump.c */
+int dcc_r_bulk(int ofd,
+               int ifd,
+               unsigned f_size,
+               enum dcc_compress compression);
+
+int dcc_pump_readwrite(int ofd, int ifd, size_t n);
+
+/* mapfile.c */
+int dcc_map_input_file(int in_fd, off_t in_size, char **buf_ret);
+
