@@ -81,7 +81,7 @@ struct dcc_hostdef _dcc_local = {
     NULL,
     4,                          /* number of tasks */
     (char *)"localhost",        /* verbatim string */
-    1,                          /* protocol (ignored) */
+    DCC_VER_1,                  /* protocol (ignored) */
     DCC_COMPRESS_NONE,          /* compression (ignored) */
     NULL    
 };
@@ -92,30 +92,37 @@ struct dcc_hostdef *dcc_hostdef_local = &_dcc_local;
 /**
  * Returns a newly allocated buffer.
  **/
-char * dcc_make_lock_filename(const char *lockname,
-                              const struct dcc_hostdef *host,
-                              int iter)
+int dcc_make_lock_filename(const char *lockname,
+                           const struct dcc_hostdef *host,
+                           int iter,
+                           char **filename_ret)
 {
     char * buf;
-    const char *tempdir;
+    int ret;
+    char *lockdir;
 
-    tempdir = dcc_get_tempdir();
+    if ((ret = dcc_get_lock_dir(&lockdir)))
+        return ret;
 
     if (host->mode == DCC_MODE_LOCAL) {
-        asprintf(&buf, "%s/lock_%s_localhost_%d", tempdir, lockname, iter);
+        if (asprintf(&buf, "%s/%s_localhost_%d", lockdir, lockname, 
+                     iter) == -1)
+            return EXIT_OUT_OF_MEMORY;
     } else if (host->mode == DCC_MODE_TCP) {
-        asprintf(&buf, "%s/lock_%s_tcp_%s_%d_%d", tempdir, lockname, host->hostname,
-                 host->port, iter);
+        if (asprintf(&buf, "%s/%s_tcp_%s_%d_%d", lockdir, lockname,
+                     host->hostname,
+                     host->port, iter) == -1)
+            return EXIT_OUT_OF_MEMORY;
     } else if (host->mode == DCC_MODE_SSH) {
-        asprintf(&buf, "%s/lock_%s_ssh_%s_%d", tempdir, lockname, host->hostname, iter);
+        if (asprintf(&buf, "%s/%s_ssh_%s_%d", lockdir, lockname,
+                     host->hostname, iter) == -1)
+            return EXIT_OUT_OF_MEMORY;
     } else {
         rs_fatal("oops");
     }
 
-    if (!buf)
-        rs_fatal("failed to allocate string");
-
-    return buf;
+    *filename_ret = buf;
+    return 0;
 }
 
 
@@ -198,7 +205,8 @@ int dcc_lock_host(const char *lockname,
     char *fname;
     int ret;
 
-    fname = dcc_make_lock_filename(lockname, host, slot);
+    if ((ret = dcc_make_lock_filename(lockname, host, slot, &fname)))
+        return ret;
 
     if ((ret = dcc_open_lockfile(fname, lock_fd)) != 0) {
         free(fname);
