@@ -102,9 +102,9 @@ int dcc_x_result_header(int ofd,
 }
 
 
-int dcc_x_cc_status(int ofd, unsigned status)
+int dcc_x_cc_status(int ofd, int status)
 {
-    return dcc_x_token_int(ofd, "STAT", status);
+    return dcc_x_token_int(ofd, "STAT", (unsigned) status);
 }
 
 
