@@ -68,6 +68,7 @@
 #ifdef HAVE_SYS_SIGNAL_H
 #  include <sys/signal.h>
 #endif /* HAVE_SYS_SIGNAL_H */
+#include <sys/param.h>
 
 #include "distcc.h"
 #include "trace.h"
@@ -159,16 +160,85 @@ int dcc_service_job(int in_fd,
 }
 
 
-static char *dcc_input_tmpnam(char * orig_input)
+static int dcc_input_tmpnam(char * orig_input,
+                            char **tmpnam_ret)
 {
     const char *input_exten;
+    
     rs_trace("input file %s", orig_input);
     input_exten = dcc_find_extension(orig_input);
     if (input_exten)
         input_exten = dcc_preproc_exten(input_exten);
     if (!input_exten)           /* previous line might return NULL */
         input_exten = ".tmp";
-    return dcc_make_tmpnam("server", input_exten);
+    return dcc_make_tmpnam("distccd", input_exten, tmpnam_ret);
+}
+
+
+
+/**
+ * Find the absolute path for the first occurrence of @p compiler_name on the
+ * PATH.  Print a warning if it looks like a symlink to distcc.
+ *
+ * We want to guard against somebody accidentally running the server with a
+ * masqueraded compiler on its $PATH.  The worst that's likely to happen here
+ * is wasting some time running a distcc or ccache client that does nothing,
+ * so it's not a big deal.  (This could be easy to do if it's on the default
+ * PATH and they start the daemon from the command line.)
+ *
+ * At the moment we don't look for the compiler too.
+ **/
+static int dcc_check_compiler_masq(char *compiler_name)
+{
+    const char *envpath, *newpath, *p, *n;
+    char *buf = NULL;
+    struct stat sb;
+    int len;
+    char linkbuf[MAXPATHLEN];
+
+    if (compiler_name[0] == '/') 
+        return 0;
+    
+    if (!(envpath = getenv("PATH"))) {
+        rs_trace("PATH seems not to be defined");
+        return 0;
+    }
+
+    for (n = p = envpath, newpath = NULL; *n; p = n) {
+        n = strchr(p, ':');
+        if (n)
+            len = n++ - p;
+        else {
+            len = strlen(p);
+            n = p + len;
+        }
+        if (asprintf(&buf, "%.*s/%s", len, p, compiler_name) == -1) {
+            rs_log_crit("asnprintf failed");
+            return EXIT_DISTCC_FAILED;
+        }
+
+        if (lstat(buf, &sb) == -1)
+            continue;           /* ENOENT, EACCESS, etc */
+        if (!S_ISLNK(sb.st_mode)) {
+            rs_trace("%s is not a symlink", buf);
+            break;              /* found it */
+        }
+        if ((len = readlink(buf, linkbuf, sizeof linkbuf)) <= 0)
+            continue;
+        linkbuf[len] = '\0';
+        
+        if (strstr(linkbuf, "distcc")) {
+            rs_log_warning("%s on distccd's path is %s and really a link to %s",
+                           compiler_name, buf, linkbuf);
+            break;              /* but use it anyhow */
+        } else {
+            rs_trace("%s is a safe symlink to %s", buf, linkbuf);
+            break;              /* found it */
+        }
+    }
+
+    free(buf);
+    return 0;
 }
 
 
@@ -188,8 +258,11 @@ static int dcc_run_job(int in_fd,
     enum dcc_protover protover;
     enum dcc_compress compr;
     
-    err_fname = dcc_make_tmpnam("cc", ".stderr");
-    out_fname = dcc_make_tmpnam("cc", ".stdout");
+    if ((ret = dcc_make_tmpnam("distcc", ".stderr", &err_fname)))
+        return ret;
+    if ((ret = dcc_make_tmpnam("distcc", ".stdout", &out_fname)))
+        return ret;
+    
     dcc_remove_if_exists(err_fname);
     dcc_remove_if_exists(out_fname);
 
@@ -213,8 +286,10 @@ static int dcc_run_job(int in_fd,
     
     rs_trace("output file %s", orig_output);
 
-    temp_i = dcc_input_tmpnam(orig_input);
-    temp_o = dcc_make_tmpnam("server", ".o");
+    if ((ret = dcc_input_tmpnam(orig_input, &temp_i)))
+        goto out_cleanup;
+    if ((ret = dcc_make_tmpnam("distccd", ".o", &temp_o)))
+        goto out_cleanup;
 
     compr = (protover == 2) ? DCC_COMPRESS_LZO1X : DCC_COMPRESS_NONE;
 
@@ -223,7 +298,11 @@ static int dcc_run_job(int in_fd,
         || (ret = dcc_set_output(argv, temp_o)))
         goto out_cleanup;
 
-    if ((compile_ret = dcc_spawn_child(argv, &cc_pid, "/dev/null", out_fname, err_fname))
+    if ((ret = dcc_check_compiler_masq(argv[0])))
+        goto out_cleanup;
+    
+    if ((compile_ret = dcc_spawn_child(argv, &cc_pid,
+                                       "/dev/null", out_fname, err_fname))
         || (compile_ret = dcc_collect_child("cc", cc_pid, &status))) {
         /* We didn't get around to finding a wait status from the actual compiler */
         status = W_EXITCODE(compile_ret, 0);
