@@ -23,5 +23,5 @@
 /* srvnet.c */
 int dcc_socket_listen(int port, int *fd, const char *listen_addr);
 int is_a_socket(int fd);
-int dcc_check_client(int fd);
+int dcc_check_client(struct sockaddr *, int);
 
