@@ -45,8 +45,6 @@
 #include "snprintf.h"
 
 
-static char *state_dir;
-
 /* Use these strings for compiler state to keep things centralized.  They're
  * essentially tokens, not freeform strings. */
 const char *STATE_COMPILE = "Compile",
@@ -93,27 +91,6 @@ const char *dcc_state_temp_prefix = "temp_";
  **/
 
 
-/**
- * Return a static pointer to the directory name for status files.
- *
- * The directory is created if it does not exist.
- **/
-int dcc_get_state_dir (char **p)
-{
-    int ret;
-    
-    if (state_dir == NULL) {
-        asprintf(&state_dir, "%s/state", dcc_get_tempdir ());
-        if ((ret = dcc_make_dir (state_dir)))
-            return ret;
-    }
-
-    *p = state_dir;
-
-    return 0;
-}
-
-
 /**
  * Return newly allocated buffer holding the name of this process's state file.
  *
