@@ -1,9 +1,8 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/tempfile.h,v 1.3 2003/06/10 05:47:31 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,12 +21,14 @@
  */
 
 /* tempfile.c */
-int dcc_mkfifo(const char *fifo_name);
-const char *dcc_get_tempdir(void);
-int dcc_setup_tempdir(void);
+int dcc_get_tempdir(const char **);
+int dcc_make_tmpnam(const char *, const char *suffix, char **);
+
+int dcc_mkdir(const char *path);
+int dcc_get_lock_dir(char **path_ret);
+int dcc_get_state_dir(char **path_ret);
+
+
+/* cleanup.c */
 void dcc_cleanup_tempfiles(void);
-char *dcc_make_tmpnam(const char *, const char *suffix);
-
-int dcc_enter_tempdir(void);
-
-int dcc_make_dir (const char *path);
+int dcc_add_cleanup(char *filename);
