@@ -32,7 +32,6 @@
 #include <netdb.h>
 
 #include <sys/stat.h>
-
 #include <sys/time.h>
 
 #ifdef HAVE_SYS_RESOURCE_H
@@ -219,17 +218,16 @@ int dcc_ignore_sigpipe(int val)
     return 0;
 }
 
-/* If we find a distcc and/or (perhaps) a ccache masqerade dir on the PATH,
- * remove all the dirs up through the last one we find.  If the compiler
- * name is specified, we just look for a distcc masquerade dir that contains
- * that name.  If compiler is NULL, look for either a distcc or a ccache
- * masquerade dir with "cc" in them (since that's a common name to include). */
+/**
+ * If we find a distcc masquerade dir on the PATH, remove all the dirs up
+ * to that point.
+ **/
 int dcc_trim_path(const char *compiler_name)
 {
     const char *envpath, *newpath, *p, *n;
     char linkbuf[MAXPATHLEN], *buf;
     struct stat sb;
-    int len, look_for_ccache_too;
+    int len;
 
     if (!(envpath = getenv("PATH"))) {
         rs_trace("PATH seems not to be defined");
@@ -238,13 +236,6 @@ int dcc_trim_path(const char *compiler_name)
 
     rs_trace("original PATH %s", envpath);
 
-    if (compiler_name)
-	look_for_ccache_too = 0;
-    else {
-	compiler_name = "cc";
-	look_for_ccache_too = 1;
-    }
-
     /* Allocate a buffer that will let us append "/cc" onto any PATH
      * element, even if there is only one item in the PATH. */
     if (!(buf = malloc(strlen(envpath)+1+strlen(compiler_name)+1))) {
@@ -270,8 +261,7 @@ int dcc_trim_path(const char *compiler_name)
         if ((len = readlink(buf, linkbuf, sizeof linkbuf)) <= 0)
             continue;
         linkbuf[len] = '\0';
-        if (strstr(linkbuf, "distcc")
-            || (look_for_ccache_too && strstr(linkbuf, "ccache"))) {
+        if (strstr(linkbuf, "distcc")) {
             /* Set newpath to the part of the PATH past our match. */
             newpath = n;
         }
