@@ -1,9 +1,8 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Id: util.h,v 1.17 2003/01/27 11:50:18 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,8 +30,12 @@ int dcc_getenv_bool(const char *name, int def_value);
 int set_cloexec_flag (int desc, int value);
 int dcc_ignore_sigpipe(int val);
 int dcc_remove_if_exists(const char *fname);
+int dcc_trim_path(const char *compiler_name);
+int dcc_set_path(const char *newpath);
+char *dcc_abspath(const char *path, int path_len);
 
 #define str_equal(a, b) (!strcmp((a), (b)))
 
 
 
+int dcc_dup_part(const char **psrc, char **pdst, const char *sep);
