@@ -158,6 +158,8 @@ Copyright (C) 2002, 2003 by Martin Pool
 
 # TODO: Test compilation over LZO
 
+# TODO: Test with DISTCC_DIR set, and not set.
+
 
 import time, sys, string, os, types, re, popen2, pprint
 import signal, os.path, string
@@ -203,7 +205,7 @@ as soon as that happens we can go ahead and start the client."""
         SimpleDistCC_Case.setup(self)
         self.daemon_pidfile = os.path.join(os.getcwd(), "daemonpid.tmp")
         self.daemon_logfile = os.path.join(os.getcwd(), "distccd.log")
-        self.server_port = random.randint(42000, 43000)
+        self.server_port = 42000 # random.randint(42000, 43000)
         self.startDaemon()
         self.setupEnv()
 
@@ -256,8 +258,16 @@ as soon as that happens we can go ahead and start the client."""
         # find out straight away.  If it starts successfully, then we
         # can go ahead and try to connect.
         
-        cmd = self.daemon_command()
-        self.runcmd(cmd)
+        while 1:
+            cmd = self.daemon_command()
+            result, out, err = self.runcmd_unchecked(cmd)
+            if result == 0:
+                break
+            elif result == EXIT_BIND_FAILED:
+                self.server_port += 1
+                continue
+            else:
+                self.fail("failed to start daemon: %d" % result)
         self.add_cleanup(self.killDaemon)
 
 
@@ -663,6 +673,8 @@ large foo!
 class NoDetachDaemon_Case(CompileHello_Case):
     """Check that without --no-detach the server goes into the background and can be stopped."""
     def startDaemon(self):
+        # FIXME: This  does not work well if it happens to get the same
+        # port as an existing server, because we can't catch the error.
         cmd = ("distccd --no-detach --daemon --verbose --log-file %s --pid-file %s --port %d --allow 127.0.0.1" %
                (self.daemon_logfile, self.daemon_pidfile, self.server_port))
         self.pid = self.runcmd_background(cmd)
